
/*
 * ToolBarDemo.java requires the following addditional files:
 * ./Back24.gif
 * ./Forward24.gif
 * ./Up24.gif
 */

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class ToolBarDemo extends JFrame implements ActionListener
{
    private final JTextArea textArea;
    private static final String PREVIOUS = "previous";
    private static final String UP = "up";
    private static final String NEXT = "next";

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    public ToolBarDemo()
    {
        //Create the toolbar and three buttons.
        final JToolBar toolBar = new JToolBar("Still draggable");
        toolBar.add(makeNavigationButton("Back24.gif", PREVIOUS,
                                         "Back to previous something-or-other",
                                         "Previous"));

        toolBar.add(makeNavigationButton("Up24.gif", UP,
                                         "Up to something-or-other",
                                         "Up"));

        toolBar.add(makeNavigationButton("Forward24.gif", NEXT,
                                         "Forward to something-or-other",
                                         "Next"));

        //Create the text area used for output.  Request
        //enough space for 5 rows and 30 columns.
        textArea = new JTextArea(5, 30);
        textArea.setEditable(false);
        JScrollPane scrollPane = new JScrollPane(textArea);

        //Lay out the main panel.
        final JPanel jp = new JPanel(new BorderLayout());
        jp.setPreferredSize(new Dimension(450, 130));
        jp.add(toolBar, BorderLayout.PAGE_START);
        jp.add(scrollPane, BorderLayout.CENTER);

        //Create and set up the window.
        final JFrame frame = new JFrame("ToolBarDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Add content to the window.
        frame.add(jp);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }


    protected JButton makeNavigationButton(String imageName,
                                           String actionCommand,
                                           String toolTipText,
                                           String altText)
    {
        //Create and initialize the button.
        JButton button = new JButton();
        button.setActionCommand(actionCommand);
        button.setToolTipText(toolTipText);
        button.addActionListener(this);
        button.setIcon(new ImageIcon(imageName));
        return button;
    }


    public void actionPerformed(ActionEvent e)
    {
        String cmd = e.getActionCommand();
        String description = null;

        // Handle each button.
        if (PREVIOUS.equals(cmd)) { //first button clicked
            description = "taken you to the previous <something>.";
        } else if (UP.equals(cmd)) { // second button clicked
            description = "taken you up one level to <something>.";
        } else if (NEXT.equals(cmd)) { // third button clicked
            description = "taken you to the next <something>.";
        }

        displayResult("If this were a real app, it would have "
                        + description);
    }


    protected void displayResult(String actionDescription) {
        textArea.append(actionDescription + "\n");
        textArea.setCaretPosition(textArea.getDocument().getLength());
    }


    public static void main(String[] args)
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                new ToolBarDemo();
            }
        });
    }
}
