/*

*/

import renderer.framebuffer.FrameBuffer;
import renderer.framebuffer.FrameBufferPanel;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   The listener object holds references to the
   important objects in our program. We use the
   listener's constructor as the way to pass this
   information to the listener object.
*/
public class InteractiveFrame_v1
{
   public static void main(String[] args)
   {
      final JFrame jf = new JFrame("Interactive Frame v1");
      final FrameBufferPanel fbp = new FrameBufferPanel(600, 600);
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);

      final Scene scene = new Scene();

      jf.addKeyListener(new KListener_v1(fbp, scene));

      final Model model = new Sphere(1.0);
      scene.addModel(model);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 2.0));
      }
      final FrameBuffer fb = fbp.getFrameBuffer();
      Pipeline.render(scene, fb);
      fbp.update();
   }
}


// A private class definition.
class KListener_v1 implements KeyListener
{
   public FrameBufferPanel fbp = null;
   public Scene scene = null;

   public KListener_v1(FrameBufferPanel fbp,
                       Scene scene)
   {
      this.fbp = fbp;
      this.scene = scene;
   }

   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
      Model model = scene.getModel(0);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 0.1));
      }
      final FrameBuffer fb = fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(scene, fb);
      fbp.update();
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

      JFrame object
+----------------------+
|                      |       KeyListener         KListener_v1 object
|  List<KeyListener>---|------>>+------+        +-------------------------+
|                      |        |      |        | FrameBufferPanel fbp    |
|                      |        +------+        | Scene scene             |
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
