/*

*/

import renderer.framebuffer.FrameBuffer;
import renderer.framebuffer.FrameBufferPanel;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   The important objects in our program that the listener
   object needs to access are now static variables in our
   program. This way the listener can see them (we are
   using a form of "global variables" which is NOT good).
*/
public class InteractiveFrame_v2
{
   public static FrameBufferPanel fbp = null;
   public static Scene scene = null;

   public static void main(String[] args)
   {
      final JFrame jf = new JFrame("Interactive Frame v2");
      fbp = new FrameBufferPanel(600, 600);
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(new KListener_v2());

      scene = new Scene();
      final Model model = new Sphere(1.0);
      scene.addModel(model);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 2.0));
      }
      final FrameBuffer fb = fbp.getFrameBuffer();
      Pipeline.render(scene, fb);
      fbp.update();
   }//main()
}


// A private class definition.
class KListener_v2 implements KeyListener
{
   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
      final Model model = InteractiveFrame_v2.scene.getModel(0);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 0.1));
      }
      final FrameBuffer fb = InteractiveFrame_v2.fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(InteractiveFrame_v2.scene, fb);
      InteractiveFrame_v2.fbp.update();
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                               InteractiveFrame_v2 class
                                               +===========================+
                                              / static FrameBufferPanel fbp \
                                              \ static Scene scene          /
                                              /                             \
      JFrame object                           \ staic main()                /
+----------------------+                       +===========================+
|                      |       KeyListener
|  List<KeyListener>---|------>>+------+
|                      |        |      |           KListener_v2 object
|                      |        +------+        +-------------------------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
