/*

*/

import renderer.framebuffer.FrameBuffer;
import renderer.framebuffer.FrameBufferPanel;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   Our program IS the listener object. The important
   objects the listener needs access to are instance
   variables of a program object. We need a main()
   method to instantiate a program object.
*/
public class InteractiveFrame_v3 implements KeyListener
{
   private FrameBufferPanel fbp = null; // Compare with InteractiveFrame_v2.
   private Scene scene = null;

   public InteractiveFrame_v3() // main() turns into a constructor!
   {
      final JFrame jf = new JFrame("Interactive Frame v3");
      fbp = new FrameBufferPanel(600, 600);
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(this); // This InteractiveFrame_v3 object is the listener.

      scene = new Scene();
      final Model model = new Sphere(1.0);
      scene.addModel(model);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 2.0));
      }
      final FrameBuffer fb = fbp.getFrameBuffer();
      Pipeline.render(scene, fb);
      fbp.update();
   }//constructor


   // Implement the KeyListener interface.
   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
      final Model model = scene.modelList.get(0);
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x, v.y, v.z - 0.1));
      }
      final FrameBuffer fb = fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(scene, fb);
      fbp.update();
   }


   public static void main(String[] args)
   {
      new InteractiveFrame_v3(); // Weird!
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

      JFrame object
+----------------------+                        InteractiveFrame_v3 object
|                      |       KeyListener      (is a KeyListener object)
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | FrameBufferPanel fbp    |
|                      |        +------+        | Scene scene             |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
