/*
   For an explanation of anonymous local classes, see

   https://docs.oracle.com/javase/tutorial/java/javaOO/anonymousclasses.html
   http://tutorials.jenkov.com/java/nested-classes.html#anonymous-classes
   https://www.geeksforgeeks.org/anonymous-inner-class-java/

   See also,
     Chapter 4, Nested Types, pages 155-176 of
     Java In A Nutshell, 6th Edition,
     By David Flanagan, Benjamin Evans,
     O'Reilly Media, 2014.
*/

public class AnonymousClassWeirdness {
   int x;
   LocalInterface ac1, ac2;

   public AnonymousClassWeirdness(int x, int y1, int y2) {
      this.x = x;
      // instantiate an anonymous class object
      this.ac1 = new LocalInterface(){
         public int f(int z) {
            return z + y1 + x;
         }
      };
      // instantiate another anonymous class object
      this.ac2 = new LocalInterface(){
         public int f(int z) {
            return z + y2 + x;
         }
      };
   }

   public static void main(String[] args) {
      // instantiate two outer class objects
      AnonymousClassWeirdness acw1 = new AnonymousClassWeirdness(5,  3, 4);
      AnonymousClassWeirdness acw2 = new AnonymousClassWeirdness(50, 3, 4);
      // get a reference to each anonymous class object
      LocalInterface ac1 = acw1.ac1;
      LocalInterface ac2 = acw1.ac2;
      LocalInterface ac3 = acw2.ac1;
      LocalInterface ac4 = acw2.ac2;
      // garbage collect(?) the outer class objects
      acw1 = null;
      acw2 = null;
      // call a method on each anonymous class object
      int i1 = ac1.f(1);
      int i2 = ac2.f(1);
      int i3 = ac3.f(1);
      int i4 = ac4.f(1);

      System.out.println("i1 = " + i1);
      System.out.println("i2 = " + i2);
      System.out.println("i3 = " + i3);
      System.out.println("i4 = " + i4);
   }
}


interface LocalInterface {
   public int f(int z);
}

/*
     AnonymousClassWeirdness obj          AnonymousClassWeirdness obj
       +------------------+                +----------------------+
       |                  |                |                      |
       |  int x = 5       |                |  int x = 50          |
       |                  |                |                      |
       |                  |                |                      |
       |                  |                |                      |
       +--/\-----------/\-+                +-/\---------------/\--+
          ||           ||                    ||               ||
          ||           ||                    || this$0        || this$0
   this$0 ||           || this$0         +---||---------+   +-||-----------+
          ||           ||                | this$0       |   | this$0       |
 +--------||----+    +-||-----------+    | captured y1=3|   | captured y2=4|
 | this$0       |    | this$0       |    | f(int z)     |   | f(int z)     |
 | captured y1=3|    | captured y2=4|    +--------------+   +--------------+
 | f(int z)     |    | f(int z)     |     anonymous obj     anonymous obj
 +--------------+    +--------------+
  anonymous obj       anonymous obj

*/
