/*
   For an explanation of inner classes, see

   https://docs.oracle.com/javase/tutorial/java/javaOO/nested.html
   http://tutorials.jenkov.com/java/nested-classes.html#non-static-nested-classes

   See also,
     Chapter 4, Nested Types, pages 155-176 of
     Java In A Nutshell, 6th Edition,
     By David Flanagan, Benjamin Evans,
     O'Reilly Media, 2014.
*/

public class InnerClassWeirdness {
   int x;

   public InnerClassWeirdness(int x) {
      this.x = x;
   }

   public class InnerClass {
      int y;

      public InnerClass(int y) {
         this.y = y;
      }

      public int f(int z) {
         return z + y + x;
      }
   }

   public static void main(String[] args) {
      // instantiate two outer class objects
      InnerClassWeirdness icw1 = new InnerClassWeirdness(5);
      InnerClassWeirdness icw2 = new InnerClassWeirdness(50);
      // instantiate four inner class objects
      InnerClass ic1 = icw1.new InnerClass(3);
      InnerClass ic2 = icw1.new InnerClass(4);
      InnerClass ic3 = icw2.new InnerClass(3);
      InnerClass ic4 = icw2.new InnerClass(4);
      // garbage collect(?) the outer class objects
      icw1 = null;
      icw2 = null;
      // call a method on each inner class object
      int i1 = ic1.f(1);
      int i2 = ic2.f(1);
      int i3 = ic3.f(1);
      int i4 = ic4.f(1);

      System.out.println("i1 = " + i1);
      System.out.println("i2 = " + i2);
      System.out.println("i3 = " + i3);
      System.out.println("i4 = " + i4);
   }
}

/*
       InnerClassWeirdness obj            InnerClassWeirdness obj
       +------------------+               +---------------------+
       |                  |               |                     |
       |  int x = 5       |               |  int x = 50         |
       |                  |               |                     |
       |                  |               |                     |
       |                  |               |                     |
       +--/\-----------/\-+               +-/\--------------/\--+
          ||           ||                   || nesting      || nesting
  nesting ||           || nesting           || link         || link
     link ||           || link          +---||--------+   +-||----------+
          ||           ||               |  this$0     |   |  this$0     |
 +--------||----+    +-||-----------+   |  int y = 3  |   |  int y = 4  |
 |  this$0      |    |  this$0      |   |  f(int z)   |   |  f(int z)   |
 |  int y = 3   |    |  int y = 4   |   +-------------+   +-------------+
 |  f(int z)    |    |  f(int z)    |   InnerClass obj    InnerClass obj
 +--------------+    +--------------+
 InnerClass obj      InnerClass obj

*/
