/*
   For an explanation of local classes, see

   https://docs.oracle.com/javase/tutorial/java/javaOO/localclasses.html
   https://docs.oracle.com/javase/tutorial/java/javaOO/whentouse.html
   http://tutorials.jenkov.com/java/nested-classes.html#local-classes
   https://www.geeksforgeeks.org/local-inner-class-java/

   See also,
     Chapter 4, Nested Types, pages 155-176 of
     Java In A Nutshell, 6th Edition,
     By David Flanagan, Benjamin Evans,
     O'Reilly Media, 2014.
*/

public class LocalClassWeirdness {
   int x;
   LocalInterface lc1, lc2;

   public LocalClassWeirdness(int x, int y1, int y2) {
      this.x = x;

      class LocalClass implements LocalInterface {
         int y;

         public LocalClass(int y) {
            this.y = y;
         }

         public int f(int z) {
            return z + y + x;
         }
      }
      // instantiate two local class objects
      this.lc1 = new LocalClass(y1);
      this.lc2 = new LocalClass(y2);
   }

   public static void main(String[] args) {
      // instantiate two outer class objects
      LocalClassWeirdness lcw1 = new LocalClassWeirdness(5,  3, 4);
      LocalClassWeirdness lcw2 = new LocalClassWeirdness(50, 3, 4);
      // get a reference to each local class object
      LocalInterface lc1 = lcw1.lc1;
      LocalInterface lc2 = lcw1.lc2;
      LocalInterface lc3 = lcw2.lc1;
      LocalInterface lc4 = lcw2.lc2;
      // garbage collect(?) the outer class objects
      lcw1 = null;
      lcw2 = null;
      // call a method on each local class object
      int i1 = lc1.f(1);
      int i2 = lc2.f(1);
      int i3 = lc3.f(1);
      int i4 = lc4.f(1);

      System.out.println("i1 = " + i1);
      System.out.println("i2 = " + i2);
      System.out.println("i3 = " + i3);
      System.out.println("i4 = " + i4);
   }
}


interface LocalInterface {
   public int f(int z);
}

/*
       LocalClassWeirdness obj            LocalClassWeirdness obj
       +------------------+               +---------------------+
       |                  |               |                     |
       |  int x = 5       |               |  int x = 50         |
       |                  |               |                     |
       |                  |               |                     |
       |                  |               |                     |
       +--/\-----------/\-+               +-/\--------------/\--+
          ||           ||                   ||              ||
          ||           ||                   || this$0       || this$0
   this$0 ||           || this$0        +---||--------+   +-||----------+
          ||           ||               |  this$0     |   |  this$0     |
 +--------||----+    +-||-----------+   |  int y = 3  |   |  int y = 4  |
 |  this$0      |    |  this$0      |   |  f(int z)   |   |  f(int z)   |
 |  int y = 3   |    |  int y = 4   |   +-------------+   +-------------+
 |  f(int z)    |    |  f(int z)    |   LocalClass obj    LocalClass obj
 +--------------+    +--------------+
 LocalClass obj      LocalClass obj

*/
