/*
   An example of an outer class with an inner class.

   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html
*/

public class OuterClass {
   private int x; // instance variable

   public OuterClass(int x) { // constructor
      this.x = x;
   }

   class InnerClass {
      private int y; // instance variable

      public InnerClass(int y) { // constructor
         this.y = y;
         System.out.println(x);
      }

      public void incXbyY() { // instance method
         x += y;
         System.out.println(x);
      }
   }

   public static void main(String[] args) {
      // Draw pictures of all these objects.
      OuterClass outerObject1 = new OuterClass(1);
      OuterClass outerObject2 = new OuterClass(2);

      OuterClass.InnerClass innerObject1 = outerObject1.new InnerClass(10);
      OuterClass.InnerClass innerObject2 = outerObject1.new InnerClass(20);
      OuterClass.InnerClass innerObject3 = outerObject2.new InnerClass(30);

      innerObject1.incXbyY();
      innerObject2.incXbyY();
      innerObject3.incXbyY();
   }
}

/*
          OuterClass object                 OuterClass object
        +-------------------+             +-------------------+
        |                   |             |                   |
        |  int x = 1        |             |  int x = 2        |
        |                   |             |                   |
        |                   |             |                   |
        |                   |             |                   |
        +--/\------------/\-+             +----------/\-------+
           ||            ||                          ||
   nesting ||            || nesting                  || nesting
      link ||            || link                     || link
           ||            ||                     +----||---------+
  +--------||-----+    +-||------------+        |  this$0       |
  |     this$0    |    |  this$0       |        |  int y = 30   |
  |  int y = 10   |    |  int y = 20   |        |               |
  |               |    |               |        +---------------+
  +---------------+    +---------------+        InnerClass object
  InnerClass object    InnerClass object

*/
