/*
   An example of an outer class with a local class.

   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html
*/

public class OuterClass_v2 {
   private static OuterInterface[] localObjects = new OuterInterface[3];
   private static int index = 0;

   private int x = 0; // instance variable

   public OuterClass_v2(int y, int z) { // constructor
      class LocalClass implements OuterInterface { // local to the constructor
         private int z; // instance variable

         public LocalClass(int z) { // constructor
            this.z = z;
         }

         public void incXbyYandZ() { // instance method of LocalClass
            x += y;  // use the constructor's local variable
            x += z;  // use the instance variable
            System.out.println(x);
         }
      }//LocalClass

      localObjects[index++] = new LocalClass(z);
   }//constructor

   public static void main(String[] args) {
      // Draw pictures of all these objects.
      OuterClass_v2 outerObject1 = new OuterClass_v2(1, 5);
      OuterClass_v2 outerObject2 = new OuterClass_v2(10, 50);
      OuterClass_v2 outerObject3 = new OuterClass_v2(100, 500);

      for (OuterInterface m : localObjects) {
         m.incXbyYandZ();
      }
   }//main()
}


interface OuterInterface {
   public void incXbyYandZ();
}

/*
         OuterClass_v2 object                OuterClass_v2 object
        +-------------------+               +-------------------+
        |                   |               |                   |
        |  int x = 0        |               |  int x = 0        |
        |                   |               |                   |
        |                   |               |                   |
        |                   |               |                   |
        +--/\---------------+               +----------/\-------+
           ||                                          ||
   nesting ||                                          || nesting
      link ||                                          || link
           ||            OuterClass_v2 object     +----||-----------+
  +--------||------+    +-------------------+     | this$0          |
  |     this$0     |    |                   |     | int z = 500     |
  | int z = 5      |    |  int x = 0        |     | captured y = 100|
  | captured y = 1 |    |                   |     | incXbyYandZ()   |
  | incXbyYandZ()  |    |                   |     +-----------------+
  +----------------+    |                   |      LocalClass object
   LocalClass object    +-------/\----------+
                                ||
                                || nesting
                                || link
                                ||
                           +----||-----------+
                           | this$0          |
                           | int z = 50      |
                           | captured y = 10 |
                           | incXbyYandZ()   |
                           +-----------------+
                            LocalClass object
*/
