/*
   An example of an outer class with an anonymous class.

   https://cscircles.cemc.uwaterloo.ca/java_visualize/
   http://pythontutor.com/java.html
*/

public class OuterClass_v3 {
   private static OuterInterface[] localObjects = new OuterInterface[3];
   private static int index = 0;

   private int x = 0; // instance variable

   public OuterClass_v3(int y, int _z) { // constructor
      localObjects[index++] = new OuterInterface() { // anonymous local class
         private int z = _z; // instance variable

         public void incXbyYandZ() { // instance method of LocalClass
            x += y;  // use the constructor's local variable
            x += z;  // use the instance variable
            System.out.println(x);
         }
      };
   }//constructor

   public static void main(String[] args) {
      // Draw pictures of all these objects.
      OuterClass_v3 outerObject1 = new OuterClass_v3(1, 5);
      OuterClass_v3 outerObject2 = new OuterClass_v3(10, 50);
      OuterClass_v3 outerObject3 = new OuterClass_v3(100, 500);

      for (OuterInterface m : localObjects) {
         m.incXbyYandZ();
      }
   }//main()
}


interface OuterInterface {
   public void incXbyYandZ();
}

/*
         OuterClass_v3 object                OuterClass_v3 object
        +-------------------+               +-------------------+
        |                   |               |                   |
        |  int x = 0        |               |  int x = 0        |
        |                   |               |                   |
        |                   |               |                   |
        |                   |               |                   |
        +--/\---------------+               +----------/\-------+
           ||                                          ||
   nesting ||                                          || nesting
      link ||                                          || link
           ||            OuterClass_v3 object     +----||-----------+
  +--------||------+    +-------------------+     | this$0          |
  |     this$0     |    |                   |     | int z = 500     |
  | int z = 5      |    |  int x = 0        |     | captured y = 100|
  | captured y = 1 |    |                   |     | incXbyYandZ()   |
  | incXbyYandZ()  |    |                   |     +-----------------+
  +----------------+    |                   |      anonymous object
   anonymous object     +-------/\----------+
                                ||
                                || nesting
                                || link
                                ||
                           +----||-----------+
                           | this$0          |
                           | int z = 50      |
                           | captured y = 10 |
                           | incXbyYandZ()   |
                           +-----------------+
                            anonymous object
*/
