/*
   Demonstrate what GLUT events "look like".

   Be sure to try a few things with this program.
   What happens when you minimize a window?
   What happens when you cover and then uncover a window?
   What happens when you pres and then hold down a key?
   What happens when you click down on the mouse, then
   move the mouse, then release the mouse? Try starting
   from both inside and outside the window.

   For more documentation, see
      http://www.opengl.org/resources/libraries/glut/spec3/node45.html

   Note: The mouse entry events are weird.
*/
#include<stdio.h>

#if defined _WIN32 || defined _WIN64
#  include <windows.h>
#endif

#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif

/* Prototype for the callback functions */
// window
void display( void );
void overlayDisplay( void );
void reshape(int, int);
void visibility(int);
void idle (void);
// keyboard
void keyboard(unsigned char, int, int);
void keyboardUp(unsigned char, int, int);
// mouse
void mouse(int, int, int, int);
void motion(int, int);
void passiveMotion(int, int);
void entry(int);
// timer
void timer(int value);
// menus
void menu(int);
void subMenu(int);
int sub_menu;

int eventNumber = 0; // count events so that is is easier to keep track of them

int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(500,500);       /* 500 x 500 pixel window */
   glutInitWindowPosition(100,100);   /* place window near top left of display */
   glutCreateWindow("Watch events");  /* window title */


   /* Register a variety of callback functions */
   //window
   glutDisplayFunc(display);
   glutOverlayDisplayFunc(overlayDisplay);
   glutReshapeFunc(reshape);
   glutVisibilityFunc(visibility);
   //glutIdleFunc(idle);            // probably want to comment this out

   // keyboard
   glutKeyboardFunc(keyboard);
   glutKeyboardUpFunc(keyboardUp);  // Not even in the documentation?

   // mouse
   glutMouseFunc(mouse);
   glutMotionFunc(motion);
   glutPassiveMotionFunc(passiveMotion); // probably want to comment this out
   glutEntryFunc(entry);

   // timer (every 5 seconds)
   glutTimerFunc(5000, timer, 1);

   // create a pop up menu (with a sub menu)
   sub_menu = glutCreateMenu(subMenu);   // register the sub menu callback function
   glutAddMenuEntry("SubAction_4", 4);   // add items to the sub menu
   glutAddMenuEntry("SubAction_5", 5);
   glutCreateMenu(menu);                 // register menu callback function
   glutAddMenuEntry("Action_1", 1);      // add items to the menu
   glutAddMenuEntry("Action_2", 2);
   glutAddMenuEntry("Action_3", 3);
   glutAddSubMenu("SubMenu", sub_menu);  // add the submenu to the menu
   glutAttachMenu(GLUT_RIGHT_BUTTON);    // register which mouse button pops up menus

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   glColor3f(1.0, 0.0, 0.0);         /* draw in red */
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0); /* 3D viewing volume */
   glMatrixMode(GL_MODELVIEW);

   glEnable(GL_DEPTH_TEST);

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/*
   Here are implementations of the callback functions.
   These implementations don't really do anything except
   give you an idea of when, and how, they get called.
*/

/* The display() callback function */
void display( void )
{
   printf("%d, We got a \"display()\" event\n", eventNumber++);
}


/* The overlayDisplay() callback function */
void overlayDisplay( void )
{
   printf("%d, We got a \"overlayDisplay()\" event\n", eventNumber++);
}


/* The reshape() callback function */
void reshape( int width, int height )
{
   printf("%d, We got a \"reshape(%d, %d)\" event\n", eventNumber++, width, height);
}


/* The visibility() callback function */
void visibility(int state)
{
   printf("%d, We got a \"visibility(%d)\" event\n", eventNumber++, state);
}


/* The idle() callback function */
void idle(void)
{
   printf("%d, We got a \"idle()\" event\n", eventNumber++);
}


/* The keyboard() callback function */
void keyboard(unsigned char key, int x, int y)
{
   // Notice that keyboard input is not buffered
   // (like it is in the C Library).
   printf("%d, We got a \"keyboard(%c, %d, %d)\" event\n", eventNumber++, key, x, y);
}


/* The keyboardUp() callback function */
void keyboardUp(unsigned char key, int x, int y)
{
   printf("%d, We got a \"keyboardUp(%c, %d, %d)\" event\n", eventNumber++, key, x, y);
}


/* The mouse() callback function */
void mouse(int button, int state, int x, int y)
{
   printf("%d, We got a \"mouse(%d, %d, %d, %d)\" event\n", eventNumber++, button, state, x, y);
}


/* The mouse motion() callback function */
void motion(int x, int y)
{
   printf("%d, We got a \"motion(%d, %d)\" event\n", eventNumber++, x, y);
}


/* The mouse passiveMotion() callback function */
void passiveMotion(int x, int y)
{
   printf("%d, We got a \"passiveMotion(%d, %d)\" event\n", eventNumber++, x, y);
}


/* The mouse entry() callback function */
void entry(int state)
{
   printf("%d, We got a \"entry(%d)\" event\n", eventNumber++, state);
}


/* The timer() callback function */
void timer(int value)
{
   static int counter = 1;
   printf("%d, We got a \"timer(%d)\" event\n", eventNumber++, value);
   // re-register the timer callback function
   glutTimerFunc(5000, timer, counter++);
}


/* The menu() callback function */
void menu(int id)
{
   printf("%d, We got a \"menu(%d)\" event\n", eventNumber++, id);
}


/* The subMenu() callback function */
void subMenu(int id)
{
   printf("%d, We got a \"subMenu(%d)\" event\n", eventNumber++, id);
}
